
(*s Production du code C. *)

open Format
open Pascal

let fmt = ref std_formatter
let print s = fprintf !fmt s

(*s Types. *)

let string_of_typ = function
  | Tinteger -> "int"
  | Tbool -> "int"

(*s Liste d'identificateurs, spars par des virgules. *)

let rec idents = function
  | [] -> assert false
  | [x] -> print "%s" x
  | x :: l -> print "%s, " x; idents l

(*s Dclarations de variables. *)

let vars bl =
  List.iter 
    (fun (vl,t) -> 
       print "%s " (string_of_typ t); idents vl;
       print ";@\n") 
    bl

(*s Paramtres de fonctions ou procdures. *)

let rec param t = function
  | [] -> assert false
  | [x] -> print "%s %s" (string_of_typ t) x
  | x :: l -> print "%s %s, " (string_of_typ t) x; param t l

let rec params = function
  | [] -> 
      ()
  | (vl,t) :: l -> 
      param t vl; 
      if l <> [] then print ", ";
      params l

(*s Oprateurs binaires et unaires. *)

let string_of_binop = function
  | Eq -> "=="
  | Noteq -> "!="
  | Gt -> ">"
  | Ge -> ">="
  | Lt -> "<"
  | Le -> "<="
  | And -> "&&"
  | Or -> "||"

let string_of_unop = function
  | Not -> "!"

(*s Expressions. *)

let rec expr = function
  | Econstant (Cinteger n) -> 
      print "%d" n
  | Econstant (Cbool b) ->
      print "%b" b
  | Evar x ->
      print "%s" x
  | Ebinop (e1, op, e2) -> 
      print "(";
      expr e1; print " %s " (string_of_binop op);
      expr e2; print ")"
  | Eunop (op, e) -> 
      print "(%s" (string_of_unop op);
      expr e; print ")"

(*s Instructions. *)

let rec instr f = function
  | Iassign (x, e) when Some x = f ->
      print "_%s = " x; expr e; print ";"
  | Iassign (x, e) ->
      print "%s = " x; expr e; print ";"
  | Icond (e, st, []) ->
      print "if ("; expr e; 
      print ") {@\n  @["; statement f st;
      print "@]@\n};"
  | Icond (e, st, st') ->
      print "if ("; expr e; 
      print ") {@\n  @["; statement f st;
      print "@]@\n} else {@\n  @["; statement f st';
      print "@]@\n};"

and statement f = function
  | [] -> ()
  | [i] -> instr f i
  | i :: il -> instr f i; print "@\n"; statement f il

(*s Dclarations de fonctions et procdures. *)

let proc_and_fun = function
  | Procedure (p, pa, vl, st) ->
      print "void %s(" p;
      params pa;
      print ") {@\n  @[";
      vars vl;
      print "@\n";
      statement None st;
      print "@]@\n}@\n@\n"
  | Function (f, pa, t, vl, st) ->
      print "%s %s(" (string_of_typ t) f;
      params pa;
      print ") {@\n  @[";
      print "%s _%s;@\n" (string_of_typ t) f;
      vars vl;
      print "@\n";
      statement (Some f) st;
      print "@\nreturn _%s;" f;
      print "@]@\n}@\n@\n"

let proc_and_funs = List.iter proc_and_fun

(*s Programme. *)

let program (id, vl, pl, st) = 
  print "/* program %s */@\n@\n" id;
  print "#define true 1@\n";
  print "#define false 0@\n@\n";
  vars vl;
  print "@\n";
  proc_and_funs pl;
  print "int main() {@\n  @[";
  statement None st;
  print "@\nexit(0);@]@\n}@\n";
  pp_print_flush !fmt ()


