
(*s Analyse lexicale. *)

{

  open Lexing
  open Parser
  open Pascal

  (*s Numro de la ligne courante. *)

  let line = ref 1

  let get_line () = !line

  (*s Erreurs lexicales. *)

  exception Illegal_character

  exception Unterminated_comment

  (*s Table des mots cls. *)

  let keywords = Hashtbl.create 97

  let _ = List.iter 
	    (fun (s,k) -> Hashtbl.add keywords s k)
	    [ "if", IF; "then", THEN; "else", ELSE; "program", PROGRAM; 
	      "procedure", PROCEDURE; "function", FUNCTION;
	      "var", VAR; "begin", BEGIN; "end", END;
	      "integer", INTEGER; "bool", BOOL;
	      "and", AND; "or", OR; "not", NOT;
	      "true", CONSTANT (Cbool true); "false", CONSTANT (Cbool false) ]

  let ident_or_keyword s =
    let s = String.lowercase s in
    try Hashtbl.find keywords s with Not_found -> IDENT s

}

(*s Expressions rgulires. *)

let letter = ['a'-'z' 'A'-'Z']
let digit = ['0'-'9']

let ident = letter (letter | digit)*

let integer = ['0'-'9']+

let space = [' ' '\t']

(*s Point d'entre du lexeur. *)

rule token = parse
  | space+ 
    { token lexbuf }
  | '\n'   
    { incr line; token lexbuf }
  | ident
    { ident_or_keyword (lexeme lexbuf) }
  | ';'
    { SEMICOLON }
  | '.'
    { DOT }
  | ','
    { COMMA }
  | ':'
    { COLON }
  | ":="
    { ASSIGN }
  | '='
    { EQ }
  | "<>"
    { NOTEQ }
  | '<'
    { LT }
  | "<="
    { LE }
  | '>'
    { GT }
  | ">="
    { GE }
  | '('
    { LPAREN }
  | ')'
    { RPAREN }
  | integer
    { CONSTANT (Cinteger (int_of_string (lexeme lexbuf))) }
  | eof 
    { EOF }
  | _
    { raise Illegal_character }

