
(*s Programme principal. *)

open Printf

(*s Utilisation. *)

let usage () =
  eprintf "usage: p2c file.pas\n";
  exit 1

(*s Fichier  compiler. *)

let file = 
  if Array.length Sys.argv = 1 then usage ();
  let f = Sys.argv.(1) in
  if not (Filename.check_suffix f ".pas") then usage ();
  if not (Sys.file_exists f) then begin 
    eprintf "p2c: %s: no such file\n" f; exit 1
  end;
  f

(*s Affichage des erreurs. *)

let report_error e = 
  eprintf "%s: %d: " file (Lexer.get_line ());
  match e with
    | Lexer.Illegal_character ->
	eprintf "illegal character\n"
    | Lexer.Unterminated_comment ->
	eprintf "unterminated comment\n"
    | Parsing.Parse_error -> 
	eprintf "syntax error\n"
    | _ ->
	eprintf "anomaly: %s\n" (Printexc.to_string e)

(*s Ouverture du fichier, analyse syntaxique puis compilation. *)

let main () =
  try
    let c = open_in file in
    let lb = Lexing.from_channel c in
    let p = Parser.program Lexer.token lb in
    close_in c;
    let cfile = (Filename.chop_suffix file ".pas") ^ ".c" in
    let c = open_out cfile in
    Compile.fmt := Format.formatter_of_out_channel c;
    Compile.program p;
    close_out c
  with e ->
    report_error e; exit 1

let _ = Printexc.catch main ()
