
/*s Analyse syntaxique. */

%{

  open Pascal

%}

/*s Dclarations des tokens. */

%token <Pascal.constant> CONSTANT
%token <string> IDENT
%token EOF PROGRAM SEMICOLON DOT VAR COLON COMMA ASSIGN 
%token PROCEDURE FUNCTION
%token INTEGER BOOL LPAREN RPAREN BEGIN END LBRAC RBRAC
%token IF THEN ELSE NOT
%token EQ NOTEQ LT LE GT GE AND OR

/*s Prcdences. */

%nonassoc THEN
%nonassoc ELSE
%left OR
%left AND
%nonassoc NOT
%nonassoc EQ NOTEQ LT LE GT GE

/*s Point d'entre. */

%type <Pascal.program> program
%start program

%%

/*s Rgles de grammaire. */

program:
| PROGRAM IDENT SEMICOLON variables proc_and_funs 
  BEGIN instrs END DOT EOF
  { ($2, $4, $5, $7) }
;

variables:
| /* epsilon */ { [] }
| VAR var_decls SEMICOLON { $2 }
;

var_decls:
| var_decl { [$1] }
| var_decls SEMICOLON var_decl { $1 @ [$3] }
;

var_decl:
|  idents COLON typ { ($1, $3) }
;

idents:
| IDENT { [$1] }
| IDENT COMMA idents { $1 :: $3 }

;

typ:
| INTEGER { Tinteger }
| BOOL    { Tbool }
;

proc_and_funs:
| /* epsilon */ { [] }
| proc_and_fun SEMICOLON proc_and_funs { $1 :: $3 }

proc_and_fun:
| PROCEDURE IDENT LPAREN var_decls RPAREN SEMICOLON 
  variables BEGIN instrs END
  { Procedure ($2, $4, $7, $9) }
| FUNCTION IDENT LPAREN var_decls RPAREN COLON typ SEMICOLON 
  variables BEGIN instrs END
  { Function ($2, $4, $7, $9, $11) }
;

block:
| instr { [$1] }
| BEGIN instrs END { $2 }
;

instrs:
| instr SEMICOLON { [$1] }
| instr SEMICOLON instrs { $1 :: $3 }
;

instr:
| IDENT ASSIGN expr { Iassign ($1, $3) }
| IF expr THEN block { Icond ($2, $4, []) }
| IF expr THEN block ELSE block { Icond ($2, $4, $6) }
;

expr:
| CONSTANT { Econstant $1 }
| IDENT { Evar $1 }
| expr EQ expr { Ebinop ($1, Eq, $3) }
| expr NOTEQ expr { Ebinop ($1, Noteq, $3) }
| expr LT expr { Ebinop ($1, Lt, $3) }
| expr LE expr { Ebinop ($1, Le, $3) }
| expr GT expr { Ebinop ($1, Gt, $3) }
| expr GE expr { Ebinop ($1, Ge, $3) }
| expr AND expr { Ebinop ($1, And, $3) }
| expr OR expr { Ebinop ($1, Or, $3) }
| LPAREN expr RPAREN { $2 }
| NOT expr { Eunop (Not, $2) }
;

